



#\name{Discrete}
#\alias{ddiscrete}
#\alias{pdiscrete}
#\alias{qdiscrete}
#\alias{rdiscrete}
#\alias{aliasmat}
#\alias{aliasmat2prob}
#\title{Discrete Distribution}
#\description{
#  These functions provide information about the discrete distribution
#  where the probability of the elements of \code{values} is proportional
#  to the values given in \code{probs}, which are normalized to sum up to
#  1.  \code{ddiscrete} gives the density, \code{pdiscrete} gives the
#  distribution function, \code{qdiscrete} gives the quantile function
#  and \code{rdiscrete} generates random deviates.
#}
#\usage{
#ddiscrete(x, probs, values = 1:length(probs))
#pdiscrete(q, probs, values = 1:length(probs))
#qdiscrete(p, probs, values = 1:length(probs))
#rdiscrete(n, probs, values = 1:length(probs),
#          method = "inverse", aliasmatrix = NULL)
#aliasmat(p)
#aliasmat2prob(aliasmatrix)
#}
#\arguments{
#  \item{x,q}{vector or array of quantiles.}
#  \item{p}{vector or array of probabilites.}
#  \item{n}{number of observations.}
#  \item{probs}{probabilities of the distribution.}
#  \item{values}{values of the distribution.}
#  \item{method}{generation method, can be \code{"inverse"} or
#    \code{"alias"}.}
#  \item{aliasmatrix}{matrix needed by alias method.}
#}
#\details{
#  For the generation of the random deviates, one can choose between the
# method \code{"inverse"} which basically makes a lookup in the vector
# of the probabilities and the method \code{"alias"}.  The latter method
#  computes an aliasmatrix by the function \code{aliasmat} which allows a
#  faster data generation once this matrix is computed.  If such a matrix
#  has already been computed, it can be passed as further argument to
#  \code{rdiscrete}. \code{aliasmat2prob} computed probabilities from a
#  given \code{aliasmatrix}.
#}
# \author{Andreas Weingessel and Friedrich Leisch}
#\examples{
## a vector of length 30 whose elements are 1 with probability 0.2
## and 2 with probability 0.8.
#rdiscrete (30, c(0.2, 0.8))

### a vector of length 100 whose elements are A, B, C, D.
### The probabilities of the four values have the relation 1:2:3:3
#rdiscrete (100, c(1,2,3,3), c("A","B","C","D"))
#
#}
#\keyword{distribution}

###### Part of the package "e107"
###### http://cran.us.r-project.org/src/contrib/e1071_1.3-5.tar.gz














rdiscrete <- function (n, probs, values = 1:length(probs), method="inverse",
                       aliasmatrix = NULL)
{
    
    if (length(probs) != length(values))
        stop("rdiscrete: probs and values must have the same length.")
    if (sum(probs < 0) > 0)
        stop("rdiscrete: probs must not contain negative values.")
    
    if (n == 1)
    {
	p <- cumsum(probs)/sum(probs)
        return (values[sum(runif(1) > p) + 1])
    }
    else
    {
        method <- pmatch(method, c("inverse", "alias"))
        if (is.na(method))
            stop("rdiscrete: unknown method.")
        if (method == 1)
        {
            p <- cumsum(probs)/sum(probs)
            l <- length(probs)
            m <- numeric(n)
            a <- runif(n)
            for (i in 1:n)
                m[i] <- sum(a[i] > p)
            return(values[m + 1])
        }
        else
        {
            if (missing(aliasmatrix))
                aliasmatrix <- aliasmat(probs)
            
            x <- sample(1:nrow(aliasmatrix), n, replace=TRUE)
            y <- runif(n)
            
            retval <- rep(0, length=n)
    
            eins <- (y <= aliasmatrix[x,1])
    
            retval[eins] <- values[aliasmatrix[x[eins],2]]
            retval[!eins] <- values[aliasmatrix[x[!eins],3]]
            
            return(retval)
        }
    }
}

aliasmat <- function(p)
{
    p <- p / sum(p)
    q <- p * (pn <- length(p))
    r <- matrix(0, nrow=pn, ncol=3)

    eps <- .Machine$double.eps
    while (sum(!is.na(q))>1)
    {
        qklein <- min((1:pn)[q<=1+eps],na.rm=TRUE)
        qgross <- max((1:pn)[q>=1-eps],na.rm=TRUE)
        r[qklein,] <- c(q[qklein],qklein,qgross)
        q[qgross] <- q[qgross] + q[qklein] - 1
        q[qklein] <- NA
    }
    qmittel <- (1:pn)[!is.na(q)]
    r[qmittel,] <-  c(q[qmittel],qmittel,qmittel)

    return(r)
}

    
aliasmat2prob <- function(aliasmatrix)
{
    p <- rep(0, length = length(unique(aliasmatrix[,2:3])))
    names(p) <- (pnames <- sort(unique(aliasmatrix[,2:3])))
    
    for(n in pnames){
        if(any(aliasmatrix[,2]==n)){
            p[n] <- aliasmatrix[aliasmatrix[,2]==n,1]
        }
        if(any(aliasmatrix[,3]==n)){
            p[n] <- p[n] + sum(1 - aliasmatrix[aliasmatrix[,3]==n,1])
        }
    }
    p<- p/length(p)
    p
}



ddiscrete <- function (x, probs, values = 1:length(probs))
{
    
    if (length(probs) != length(values))
        stop("ddiscrete: probs and values must have the same length.")
    if (sum(probs < 0) > 0)
        stop("ddiscrete: probs must not contain negative values.")
    if (!is.array(x) && !is.vector(x) && !is.factor(x))
        stop("ddiscrete: x must be an array or a vector or a factor.")
    
    p <- probs/sum(probs)
    
    y <- as.vector(x)
    l <- length(y)
    z <- rep(0,l)
    
    for (i in 1:l)
        if (any(values == y[i]))
            z[i] <- p[values == y[i]]
    
    z <- as.numeric(z)
    if (is.array(x))
        dim(z) <- dim(x)
    
    return(z)
}


pdiscrete <- function (q, probs, values = 1:length(probs))
{
    
    if (length(probs) != length(values))
        stop("pdiscrete: probs and values must have the same length.")
    if (sum(probs < 0) > 0)
        stop("pdiscrete: probs must not contain negative values.")
    if (!is.array(q) & !is.vector(q))
        stop("pdiscrete: q must be an array or a vector")
    
    p <- probs/sum(probs)
    
    y <- as.vector(q)
    l <- length(y)
    z <- rep(0,l)
    
    for (i in 1:l)
        z[i] <- sum(p[values <= y[i]])
    
    z <- as.numeric(z)
    if (is.array(q))
        dim(z) <- dim(q)
    
    return(z)
}

qdiscrete <- function (p, probs, values = 1:length(probs))
{
    
    if (length(probs) != length(values))
        stop("qdiscrete: probs and values must have the same length.")
    if (sum(probs < 0) > 0)
        stop("qdiscrete: probs must not contain negative values.")
    if (!is.array(p) & !is.vector(p))
        stop("qdiscrete: p must be an array or a vector")
    
    probs <- cumsum(probs)/sum(probs)
    
    y <- as.vector(p)
    l <- length(y)
    z <- rep(0,l)
    
    for (i in 1:l)
        z[i] <- length(values) - sum(y[i] <= probs) + 1
    
    z <- as.numeric(z)
    z <- values[z]
    if (is.array(p))
        dim(z) <- dim(p)
    
    return(z)
  }




